"""
Functions that might be useful to test code and assignments in a Compilers
course.

Some functions might also be useful elsewhere (e.g.. regex testing).
"""
from .regex import check_regex, RegexTester
from .regex_crosswords import (
    regex_crossword_table, check_regex_crossword, CROSSWORD_EXAMPLES,
)
