#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['maestro',
 'maestro.classroom_db',
 'maestro.cli',
 'maestro.course',
 'maestro.course.compilers',
 'maestro.regex',
 'maestro.test',
 'maestro.types',
 'maestro.utils']

package_data = \
{'': ['*'], 'maestro': ['grader/*']}

install_requires = \
['pytest >= 4.0',
 'click >= 7.0',
 'hyperpython ~= 1.1.1',
 'sidekick ~= 0.5.2',
 'unidecode',
 'toml']

entry_points = \
{'console_scripts': ['maestro = maestro.cli:main']}

setup(name='maestro-tools',
      version='0.1.0',
      description='Maestro is a collection of tools to aid teachers in programming-based ',
      author='Fábio Macêdo Mendes',
      author_email='fabiomacedomendes@gmail.com',
      url='http://github.com/fabiommendes/maestro/',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
     )
